/*******************************************************************************
 * File name: IntelJ3.cpp
 * Description: Intel Flash burner
 * Author: Shiva	
 * Revision history:
 * 2010/10/01	Create this file.
 * ****************************************************************************/

#include "IntelJ3.h"
using namespace std;

#ifdef __MINGW32__
SOCKET sock;
#else
int sock;
#endif
int fast_mode = 0;
int debug_level = 0;  
FILE *log = stdout;                  /* log file */

void check(unsigned data)
{
  if (data & 2)
  {
    fprintf(log, "\nFlash block locked.\n");
    fflush(log);
    terminate();
    exit(1);
  }
}

/* read flash id */
void Flash_ReadID(unsigned flash_type, unsigned base, unsigned BusWidth, unsigned* DDI1, unsigned* DDI2)
{
    if (flash_type == Intel_Type)
    {
        switch(BusWidth)
        {
        case FLASH_SINGLE: // 8 bit
            outw(base,0x90);
            *DDI1 = inw(base+0x00);
            *DDI2 = inw(base+0x01);
            outw(base,0xff);
            break;
        case FLASH_DOUBLE: // double 16 bit
            outw(base,0x9090);
            *DDI1 = inw(base+0x00);
            *DDI2 = inw(base+0x02);
            outw(base,0xffff);
            break;
        case FLASH_FOUR:// four 32 bit
            outw(base,0x00500050);
            outw(base,0x00900090);
            *DDI1 = inw(base+0x00);
            *DDI2 = inw(base+0x04);
            outw(base,0x00ff00ff);
            break;
        }
    }
}

void Flash_Reset(unsigned base, unsigned BusWidth)
{
    switch(BusWidth)
    {
    case FLASH_SINGLE:
        outw(base,0xf0);
        break;
    case FLASH_DOUBLE:
        outw(base,0xf0f0);
        break;
    case FLASH_FOUR:
        //outw(base,0xf0f0f0f0);
        outw(base,0xffffffff);
        break;
    }
}
void Flash_UnLock(unsigned flash_type, unsigned base, unsigned BusWidth)
{
    unsigned data;

    if (flash_type == Intel_Type)
    {
        // Only erase the first sector
        switch(BusWidth)
        {
        case FLASH_SINGLE: // 8 bit
            outw(base,0x50);
            outw(base,0x60);
            outw(base,0xd0);

            // read status register
            outw(base,0x70);
            data = inw(base);

            while (data != 0x80) {
               outw(base,0x70);
               data=inw(base);
            }

            break;
        case FLASH_DOUBLE: // double 16 bit
            outw(base,0x5050);
            outw(base,0x6060);
            outw(base,0xd0d0);

            // read status register
            outw(base,0x7070);
            data = inw(base);

            while (data != 0x8080) {
               outw(base,0x7070);
               data=inw(base);
            }

            break;

        case FLASH_FOUR:// four 32 bit
            outw(base,0x00500050);
            outw(base,0x00600060);
            outw(base,0x00d000d0);

            // read status register
            outw(base,0x00700070);
            data = inw(base);
            while (data != 0x00800080) {
               outw(base,0x00700070);
               data=inw(base);
            }
            break;
        }
    }
}


void Flash_Lock(unsigned flash_type, unsigned base, unsigned BusWidth)
{
    unsigned data;

    if (flash_type == Intel_Type)
    {
        // Only erase the first sector
        switch(BusWidth)
        {
        case FLASH_SINGLE: // 8 bit
            outw(base,0x60);
            outw(base,0x01);

            // read status register
            outw(base,0x70);
            data = inw(base);

            while (data != 0x80) {
               outw(base,0x70);
               data=inw(base);
            }

            break;
        case FLASH_DOUBLE: // double 16 bit
            outw(base,0x6060);
            outw(base,0x0101);

            // read status register
            outw(base,0x7070);
            data = inw(base);

            while (data != 0x8080) {
               outw(base,0x7070);
               data=inw(base);
            }

            break;

        case FLASH_FOUR:// four 32 bit
            outw(base,0x00600060);
            outw(base,0x00010001);

            // read status register
            outw(base,0x00700070);
            data = inw(base);
            while (data != 0x00800080) {
            //while ( (data&0xff) != 0x80) {
               outw(base,0x00700070);
               data=inw(base);
            }
            break;
        }
    }
}


void Flash_ChipErase(unsigned flash_type, unsigned base, unsigned BusWidth)
{
    unsigned data;
    unsigned time = 0;

    if (flash_type == Intel_Type)
    {
        // Only erase the first sector
        switch(BusWidth)
        {
        case FLASH_SINGLE: // 8 bit
            outw(base,0x20);
            outw(base,0xd0);

            outw(base,0xff);
            // read status register
            outw(base,0x70);
            data = inw(base);

            while (data != 0x80) {
               outw(base,0x70);
               data=inw(base);
            }
            outw(base,0xff);

            break;
        case FLASH_DOUBLE: // double 16 bit
            outw(base,0x2020);
            outw(base,0xd0d0);

            outw(base,0xffff);
            // read status register
            outw(base,0x7070);
            data = inw(base);

            while (data != 0x8080) {
               outw(base,0x7070);
               data=inw(base);
            }
            outw(base,0xffff);

            break;

        case FLASH_FOUR:// four 32 bit
            outw(base,0x00200020);
            outw(base,0x00d000d0);

            outw(base,0x00ff00ff);
            // read status register
            outw(base,0x00700070);
            data = inw(base);
            while (data != 0x00800080) {
            //while ( (data&0xff) != 0x80) {
               outw(base,0x00700070);
               data=inw(base);
               if (time > 1000) check(data);
               time++;
            }
            outw(base,0x00ff00ff);
            break;
        }
    }
}

// this function only use in FLASH_SINGLE
void Flash_ProgramByte(unsigned flash_type, unsigned base, unsigned address, UINT8 data)
{
    volatile unsigned int s_reg = 0;

    if (flash_type == Intel_Type)
    {
       outw(base,0x40);
       outw(address,data);

       outw(base,0x70);
       s_reg=inw(base);

        while ((s_reg&0xff) != 0x80) {
          outw(base,0x70);
          s_reg=inw(base);
       }
       outw(base,0xffffffff);
    }
}

// this function only use in FLASH_DOUBLE
void Flash_ProgramHalfWord(unsigned flash_type, unsigned base, unsigned address, UINT16 data)
{
    volatile unsigned int s_reg = 0;

    if (flash_type == Intel_Type)
    {
       outw(base,0x4040);
       outw(address,data);

       outw(base,0x7070);
       s_reg=inw(base);

       while ((s_reg&0xff) != 0x80) {
          outw(base,0x7070);
          s_reg=inw(base);
       }

       outw(base,0xffff);
    }
}
// this function only use in FLASH_FOUR
void Flash_ProgramWord(unsigned flash_type, unsigned base, volatile unsigned address, volatile unsigned data)
{
    volatile unsigned int s_reg = 0;
#ifndef FPGA 
    if (flash_type == Intel_Type)
#endif // #ifndef FPGA
    {
       outw(base, 0x00400040);
       outw(address, data);

       outw(base, 0x00700070);
       s_reg = inw(base);

       while (s_reg != 0x00800080)
       {
          outw(base, 0x00700070);
          s_reg = inw(base);
       }
       outw(base, 0x00ff00ff);
    }
}
/* only for intel flash */
/* max is 16 words in one time */
void Flash_ProgramMultiWord(unsigned flash_type, unsigned base, volatile unsigned address, unsigned char *buffer, unsigned size)
{
    volatile unsigned int s_reg = 0;
    volatile unsigned int block_addr = (address/BLK_SIZE)*BLK_SIZE;
    volatile unsigned int bytes = size % 4;
    volatile unsigned int word;
    volatile unsigned int time = 0;
 
    /* not word align */
    if (size % 4 != 0) size = size - bytes;
    word = (size/4) - 1;
    word = (word << 16) + word;

    if (flash_type == Intel_Type)
    {
       outw(block_addr, 0x00e800e8);
       outw(block_addr, word);
#if 0
       for(unsigned int i = 0; i < (size/4); i++) 
       {
         data = ((((unsigned int)buffer[4*i+3]<< 24) & 0xFF000000) |
                 (((unsigned int)buffer[4*i+2]<< 16) & 0x00FF0000) |
                 (((unsigned int)buffer[4*i+1]<< 8) & 0x0000FF00) |
                 (((unsigned int)buffer[4*i]<< 0) & 0x000000FF));
         outw(address + 4*i, data);
       }
#else
       fastout(address, size, (char*)buffer);
#endif
       outw(block_addr, 0x00d000d0);
       s_reg = 0;

       if (fast_mode == 0)
       {
         while (s_reg != 0x00800080)
         {
            outw(base, 0x00700070);
            s_reg = inw(base);
            if (time > 1000) check(s_reg);
            time++;
         }
       }
       outw(base, 0x00ff00ff);

       if (bytes)
       {
         word = 0;
         for (unsigned int j = 0; j <= bytes; j++) word = word | (buffer[size+j] << (j*8));
         Flash_ProgramWord (flash_type, base, address+size, word);
       }
    }
    else printf("not intel flash !!\n");
}

/* burn image to flash */
int Burn_image(unsigned flash_type, unsigned int address, unsigned char* start, unsigned int size, unsigned int base)
{
  unsigned int end = address + size;
  unsigned int data;
  unsigned int buf_size;
  unsigned char temp[64];

  while (address < end)
  {
    if (flash_type == Intel_Type)
    {
      buf_size = ((end - address) > 64) ? 64 : (end - address);
      for (unsigned i = 0; i < buf_size; i++) temp[i] = *(start++);
      Flash_ProgramMultiWord(flash_type, base, address, temp, buf_size);
      address = address + buf_size;
    }
    else
    { 
      for (unsigned i = 0; i < 4; i++) temp[i] = *(start++);

      data = ((((unsigned int)temp[3]<< 24) & 0xFF000000) |
              (((unsigned int)temp[2]<< 16) & 0x00FF0000) |
              (((unsigned int)temp[1]<< 8) & 0x0000FF00) |
              (((unsigned int)temp[0]<< 0) & 0x000000FF));

      Flash_ProgramWord(flash_type, base, address, data);
      address = address + 4;
    }

    if (address % 8192 == 0)
    {
      fprintf(log, ".");
      fflush(log);
    }
  }
  return 0;
}

unsigned int usage(void)
{
  printf( "--port:\t\tSpecific port number to connect with ICEman (default to 2354)\n");
  printf( "--base:\t\tSpecific flash base address (default to 0x8040000)\n");
  printf( "--addr:\t\tSpecific flash target address to write (default to 0x8040000)\n");
  printf( "--image:\tSpecific image name to burn\n");
  printf( "--preserve:\tSpecific to preserve the content in first and last erase blocks\n");
  printf( "--log:\t\tSpecific the log file to store output message (default to stdout)\n");
  printf( "--reset-target:\ttarget reset.\n");
  printf( "--target:\tSpecific board type to map ROM start address to 0x0 (xc5/ag101/ag101p_16mb)\n");
  printf( "--verify:\tVerify after flash burning\n");
  printf( "--version:\tShow flash burning version\n");
  printf( "--unlock:\tUnlock flash before burning\n");
  printf( "--lock:\t\tLock flash after burning\n");
  printf( "--fast:\t\tBurning flash in fast mode\n");
  printf( "--erase-all:\tErase entire flash before burning\n");
  printf( "--help:\t\tThe usage for IntelJ3 Burner\n");
  return 0;
}

int main(int argc, char **argv)
{
  int c = 0;
  unsigned int port = 2354;         /* default connecting port */
  unsigned int base = 0x80400000;    /* flash base address */
  unsigned int address = 0x80400000; /* default write to flash base */
  unsigned int smc_base = 0x80400000;
  unsigned int size;                 /* image size */
  unsigned int erase_size;           /* erase size */
  unsigned int end;                  /* binary end address */
  unsigned int pt, i, j;
  unsigned int id1, id2;             /* flash id */ 
  unsigned int first_size, last_size;             
  unsigned char* flash = NULL;       /* image pointer */
  unsigned char* start = NULL;       /* image pointer */
  unsigned char first[BLK_SIZE], last[BLK_SIZE];
  FILE *image = NULL;                /* image to burn */
  char flash_type = Intel_Type;
  char cmd = 0;
  char *name = new char[1024];
  int preserve = 0;
  int specify_base = 0;
  int specify_address = 0;
  int xc5_enable = 0;
  int ag301p_enable = 0;
  int ag101_enable = 0;
  int p24_enable = 0;
  int verify = 0;
  int lock = 0;
  int unlock = 0;
  int erase_all = 0;
  int verify_only = 0;
  int force = 0;

  /* parameter handler */
  while(1)
  {
    int option_index;
    c = getopt_long(argc, argv, "uzgyvcVtoehFl:x:k:p:a:b:f:i", long_option, &option_index);
    if (c == EOF) break;

    switch (c)
    {
      case 'u':
	erase_all = 1;
	break;
      case 'z':
	lock = 1;
	break;
      case 'g':
	fast_mode = 1;
	break;
      case 'y':
	unlock = 1;
	break;
      case 'v':
	verify = 1;
	break;
      case 'c':
        verify_only = 1;
        break;
      case 'l':
	if ((log = fopen(optarg, "w"))== NULL) {
          fprintf (log, "Error: can't open file %s\n", optarg);
          fflush(log);
          exit(1);
        }
        break;
      case 's':
        fprintf (log, "Branch%s\n", BRANCH_NAME);
        fprintf (log, "%s\n", COMMIT_ID);
        fflush(log);
        exit(0);
      case 'V':
        fprintf (log, "IntelJ3 Burner BUILD_ID: %d\n", BUILD_ID);
        fflush(log);
        exit(0);
	break;
      case 'x':
        if (strncmp(optarg, "xc5", 6) == 0)
        {
          xc5_enable = 1;
        } else if (strncmp(optarg, "ag101", 6) == 0)
        {
  	  ag101_enable = 1;
        } else if (strncmp(optarg, "ag301p", 6) == 0)
        {
  	  ag301p_enable = 1;
        } else if (strncmp(optarg, "ag101p_16mb", 6) == 0)
        {
  	  p24_enable = 1;
        }
        break;
      case 't':
        cmd = RESET_TARGET;
        break;
      case 'o':
        cmd = RESET_HOLD;
        break;
      case 'e':
        preserve = 1;
        break;
      case 'p':
        port = strtol(optarg, NULL, 0);
        break;
      case 'a':
        sscanf(optarg,"0x%x", &address);
        specify_address = 1;
        break;
      case 'b':
        sscanf(optarg,"0x%x", &base);
        specify_base = 1;
        break;
      case 'f':
        if (strncmp(optarg, "mx", 6) == 0)
        {
          flash_type = MX_Type;
        } else if (strncmp(optarg, "sst", 6) == 0)
        {
          flash_type = SST_Type;
        } else if (strncmp(optarg, "intel", 6) == 0)
        {
          flash_type = Intel_Type;
        } else if (strncmp(optarg, "amd", 6) == 0)
        {
          flash_type = AMD_Type;
        }
        break;
      case 'i':
        if (strncmp(optarg, "/cygdrive/", 10) == 0)
        {
          name[0] = optarg[10];
          name[1] = ':';
          name[2] = '\0';
          strcat(name, optarg + 11);
        }
        else name = optarg;
	if ((image = fopen(name, "rb"))== NULL) {
                fprintf (log, "Error: can't open file %s\n", optarg);
                fflush(log);
                exit(1);
        }
        break;
      case 'F':
        force = 1;
        break;
      case 'h':
      case '?':
      default:
        usage();
        exit(0);
        break;
    }
  }

  fprintf (log, "IntelJ3 Burner BUILD_ID: %d\n", BUILD_ID);
  fflush(log);

  initial_socket(port);
#ifdef __MINGW32__
  Sleep (200);
#else
  usleep (200000);
#endif
  send_cmd(RESET_AICE);

  if (cmd != 0)
  {
    send_cmd(cmd);
    terminate();
    exit(0);
  }

  if (xc5_enable)
  {
    send_cmd(HOLD_CORE);
    smc_base = xc5();
  }
  else if (ag101_enable)
  {
    send_cmd(HOLD_CORE);
    smc_base = ag101();
  }
  else if (ag301p_enable)
  {
    send_cmd(HOLD_CORE);
    smc_base = ag301p();
  }
  else if (p24_enable)
  {
    send_cmd(HOLD_CORE);
    smc_base = p24();
  }
  else
  {
    fprintf (log, "WARNING: No target specified.  ");
    if (specify_base == 0)
      fprintf (log, "Use 0x80400000 as base address.\n");
    else
      fprintf (log, "Use 0x%8x as base address.\n", base);
  }

  if (specify_address == 0) address = smc_base;
  if (specify_base == 0) base = smc_base;

  flash = get_image(image, &size);
  start = flash;

  /* handle signal */
  signal(SIGTERM, handle_int);
  signal(SIGINT, handle_int);
#ifndef __MINGW32__
  signal(SIGKILL, handle_int);
#endif
  /* calculate address */
  end = address + size;
  pt = (address/BLK_SIZE) * BLK_SIZE;
  erase_size = ((size/BLK_SIZE) + 1) * BLK_SIZE;

  /* check device ID */
  if (force == 0)
  {
    Flash_ReadID(flash_type, base, FLASH_FOUR, &id1, &id2);
    if (id1 == 0x890089 && id2 == 0x180018) {
      fprintf(log, "burn data to intel flash\n");
      fflush(log);
    }
    else
    {
      terminate();
      fprintf(log, "Not intel flash !!\n");
      fflush(log);
      return 0;
    }
  }

  /* verify only */
  if (verify_only)
  {
    fprintf(log, "\nVerifying...\n");
    fflush(log);
    if (Verification(address, start, size))
    {
      fprintf(log, "Verify fail !!!\n");
      fflush(log);
    }
    else 
    {
      fprintf(log, "\nVerify success.\n");
      fflush(log);
    }

    terminate();
    return 0;
  }

  /* unlock flash */
  if (unlock)
  {
    fprintf(log, "unlocking");
    fflush(log);
    for (i = 0, j = pt; i < erase_size/BLK_SIZE; i++)
    {
      fprintf(log, ".");
      fflush(log);
      Flash_UnLock(flash_type, j, FLASH_FOUR);
      j += BLK_SIZE;
    }
    fprintf(log, "\n");
    fflush(log);
  }
  /* record the content in first and last erase block */
  if (preserve)
  {
    first_size = address - pt;
    last_size = erase_size - size - first_size;

    if (first_size)
    {
      Record_content(pt, first, first_size);
    }
    if (last_size)
    {
      Record_content(end, last, last_size);
    }
  }
  /* block erase */
  if (erase_all)
  {
    pt = base;
    erase_size = FLASH_SIZE;
    if (p24_enable) erase_size = 0x100000;
  }
  fprintf(log, "erase from address = %x\n", pt);
  fflush(log);
  for (i = 0, j = pt; i < erase_size/BLK_SIZE; i++)
  {
    fprintf(log, "erasing block %d (0x%x ~ 0x%x)\n", i+1, j, j + BLK_SIZE);
    fflush(log);
    Flash_ChipErase(flash_type, j, FLASH_FOUR);
    j += BLK_SIZE;
  }
  if (debug_level >= 3)
  {
    fprintf(log, "address = %x, base = %x, flash type = %d, size = %x\n", address, base, flash_type, size); 
    fflush(log);
  }
  fprintf(log, "burn flash from 0x%x to 0x%x\n", address ,end);
  fflush(log);
  fprintf(log, "burning");
  fflush(log);
  /* burn image */
  Burn_image(flash_type, address, flash, size, base);
  /* burn first and last block content back to flash */

  if (preserve)
  {
    if (first_size)
      Burn_image(flash_type, pt, first, first_size, base);
    if (last_size)
      Burn_image(flash_type, end, last, last_size, base);
  }
  /* verify */
  if (verify)
  {
    fprintf(log, "\nVerifying...\n");
    fflush(log);
    if (Verification(address, start, size))
    {
      fprintf(log, "Verify fail !!!\n");
      fprintf(log, "Please reburn again.\n");
      fflush(log);
    }
    else 
    {
      fprintf(log, "\nVerify success.\n");
      fprintf(log, "Flash burning done.\n");
      fflush(log);
    }
  }
  else
  {
    fprintf(log, "\nFlash burning done.\n");
    fflush(log);
  } 

  /* lock flash */
  if (lock)
  {
    fprintf(log, "locking");
    fflush(log);
    for (i = 0, j = pt; i < erase_size/BLK_SIZE; i++)
    {
      fprintf(log, ".");
      fflush(log);
      Flash_Lock(flash_type, j, FLASH_FOUR);
      j += BLK_SIZE;
    }
    fprintf(log, "\n");
    fflush(log);
  }
 
  /* send terminate signal */
  terminate();
  return 0;
}

