/*******************************************************************************
 * File name: util.cpp
 * Description: Intel Flash burner
 * Author: Shiva	
 * Revision history:
 * 2010/10/01	Create this file.
 * ****************************************************************************/

#include "util.h"

#ifdef __MINGW32__
extern SOCKET sock;
#else
extern int sock;
#endif
extern int fast_mode;
extern int debug_level;  
extern FILE *log;                  /* log file */

/* send command to iceman */
int send_cmd(char cmd)
{
  char send_data[2],recv_data[2];
  send_data[0] = cmd;

  SEND(sock, send_data, 1);
  RECV(sock, recv_data, 2);

  if (recv_data[0] == cmd && recv_data[1] == 0) 
  {
    return int(recv_data[1]);
  }
  else 
  {
    return -1;
  }
}

/* write word */
/* could use outw write word to target memory */
int outw(unsigned int address, unsigned int data)
{
  char send_data[10],recv_data[2];
  send_data[0] = 0x1a;
  send_data[2] = (char)((address & 0x000000FF) >> 0);
  send_data[3] = (char)((address & 0x0000FF00) >> 8);
  send_data[4] = (char)((address & 0x00FF0000) >> 16);
  send_data[5] = (char)((address & 0xFF000000) >> 24);
  send_data[6] = (char)((data & 0x000000FF) >> 0);
  send_data[7] = (char)((data & 0x0000FF00) >> 8);
  send_data[8] = (char)((data & 0x00FF0000) >> 16);
  send_data[9] = (char)((data & 0xFF000000) >> 24);

  SEND(sock, send_data, 10);
  RECV(sock, recv_data, 2);

  if (recv_data[0] == 0x1a) return int(recv_data[1]);
  else 
  {
    return -1;
  }
}

/* fast read */
/* could use fastin read mult-word from target memory */
int fastin(unsigned int address, unsigned int size, char * buffer)
{
  char send_data[10],recv_data[2048];
  unsigned int i;
  send_data[0] = 0x1c;
  send_data[2] = (char)((address & 0x000000FF) >> 0);
  send_data[3] = (char)((address & 0x0000FF00) >> 8);
  send_data[4] = (char)((address & 0x00FF0000) >> 16);
  send_data[5] = (char)((address & 0xFF000000) >> 24);
  send_data[6] = (char)((size & 0x000000FF) >> 0);
  send_data[7] = (char)((size & 0x0000FF00) >> 8);
  send_data[8] = (char)((size & 0x00FF0000) >> 16);
  send_data[9] = (char)((size & 0xFF000000) >> 24);

  SEND(sock, send_data, 10);
  RECV(sock, recv_data, size + 2);

  for (i = 0; i < size; i++) buffer[i] = recv_data[2+i];

  if (recv_data[0] == 0x1c) return 0;
  else 
  {
    return 0xffffffff;
  }
}
/* fast write */
/* could use fastout write mult-word from target memory */
int fastout(unsigned int address, unsigned int size, char * buffer)
{
  char send_data[2048],recv_data[2];
  unsigned int i;
  send_data[0] = 0x20;
  send_data[2] = (char)((address & 0x000000FF) >> 0);
  send_data[3] = (char)((address & 0x0000FF00) >> 8);
  send_data[4] = (char)((address & 0x00FF0000) >> 16);
  send_data[5] = (char)((address & 0xFF000000) >> 24);
  send_data[6] = (char)((size & 0x000000FF) >> 0);
  send_data[7] = (char)((size & 0x0000FF00) >> 8);
  send_data[8] = (char)((size & 0x00FF0000) >> 16);
  send_data[9] = (char)((size & 0xFF000000) >> 24);

  for (i = 0; i < size; i++) send_data[i+12] = buffer[i];

  SEND(sock, send_data, size+12);
  RECV(sock, recv_data, 2);

  if (recv_data[0] == 0x20) return 0;
  else 
  {
    return 0xffffffff;
  }
}

/* read word */
/* could use inw read word from target memory */
unsigned int inw(unsigned int address)
{
  unsigned int data;
  char send_data[6],recv_data[6];
  send_data[0] = 0x1b;
  send_data[2] = (char)((address & 0x000000FF) >> 0);
  send_data[3] = (char)((address & 0x0000FF00) >> 8);
  send_data[4] = (char)((address & 0x00FF0000) >> 16);
  send_data[5] = (char)((address & 0xFF000000) >> 24);

  SEND(sock, send_data, 6);
  RECV(sock, recv_data, 6);

  data = ((((unsigned int)recv_data[2]<< 24) & 0xFF000000) |
          (((unsigned int)recv_data[3]<< 16) & 0x00FF0000) |
          (((unsigned int)recv_data[4]<< 8) & 0x0000FF00) |
          (((unsigned int)recv_data[5]<< 0) & 0x000000FF));

  if (recv_data[0] == 0x1b) return data;
  else 
  {
    return 0xffffffff;
  }
}

/* record the memory content to buffer */
int Record_content(unsigned int address, unsigned char* buffer, unsigned int size)
{
  char content[2048];
  unsigned int buffer_size, i, j = 0;

//  printf("record_content from 0x%x to 0x%x\n", address, address + size);
  while (size > 0)
  {
    if (size > 1024) buffer_size = 1024;
    else buffer_size = size;

    fastin(address, buffer_size, content);
    for (i = 0; i < buffer_size; i++)
    {
      buffer[j++] = (unsigned char)content[i];
    }
    size = size - buffer_size;
    address = address + buffer_size;
  }
  return 0;
}

/* verify after burn */
/* return 1 if varify fail */
int Verification(unsigned int address, unsigned char* start, unsigned int size)
{
  char content[2048];
  unsigned int buffer_size, i;

  while (size > 0)
  {
    if (size > 1024) buffer_size = 1024;
    else buffer_size = size;
    fastin(address, buffer_size, content);
    for (i = 0; i < buffer_size; i++)
    {
      //printf("%x: %x | %x\n", address + i, (unsigned char)content[i], *(start));
      if ((unsigned char)content[i] != *(start++)) return 1;
    }
    size = size - buffer_size;
    address = address + buffer_size;
  }
  return 0;
}

unsigned int xc5(void)
{
  const unsigned int AHBC_BASE = 0x90100000;
  const unsigned int SMC_BASE  = 0x90200000;
  unsigned int ahbc_control_reg;
  unsigned int base_address_high; // [31:20]
  unsigned int base_address_low;  // [27:15]

  /* read AHBC control register. If bit0 is 0, un-remap. If bit0 is 1, remap. */
  ahbc_control_reg = inw(AHBC_BASE + 0x88);
  if ((ahbc_control_reg & 0x1) == 0)
  {
    /* Write memory bank 0 configuration register */
    /* Disable write-protect, set BNK_SIZE to 64MB, set BNK_MBW to 32 */
    outw(SMC_BASE, 0x10000062);

    return 0; // un-remap, base is 0x0
  }

  base_address_high = inw(AHBC_BASE + 0x10);
  base_address_low = inw(SMC_BASE);

  return (base_address_high & 0xFFF00000) | (base_address_low & 0x0FFF8000);
}
/* for 24 bit platform */
unsigned int p24(void)
{
  const unsigned int AHBC_BASE = 0x00E00000;
  const unsigned int SMC_BASE  = 0x00E01000;
  unsigned int ahbc_control_reg;
  unsigned int base_address_high; // [31:20]
  unsigned int base_address_low;  // [27:15]

  /* read AHBC control register. If bit0 is 0, un-remap. If bit0 is 1, remap. */
  ahbc_control_reg = inw(AHBC_BASE + 0x88);
  //fprintf (log, "ahbc control: 0x%x\n", ahbc_control_reg);
  if ((ahbc_control_reg & 0x1) == 0)
  {
    /* Write memory bank 0 configuration register */
    /* Disable write-protect, set BNK_SIZE to 32MB, set BNK_MBW to 32 */
    outw(SMC_BASE, 0x10000052);

    return 0; // un-remap, base is 0x0
  }

  base_address_high = inw(AHBC_BASE + 0x10);
  base_address_low = inw(SMC_BASE);

  //fprintf (log, "base: 0x%x\n", ((base_address_high & 0xFFF00000) | (base_address_low & 0x0FFF8000)) >> 8);
  return ((base_address_high & 0xFFF00000) | (base_address_low & 0x0FFF8000)) & 0x00FFFFFF;
}

unsigned int ag301p(void)
{
  if (inw(0x90c00018) != 0x10080000) outw(0x90c00018, 0x10080000);
  if (inw(0x90200000) != 0x10000052) outw(0x90200000, 0x10000052);
  if (inw(0x90c00010) != 0x00080000) outw(0x90c00010, 0x00080000);

  return 0x0;
}

unsigned int ag101(void)
{
  unsigned int data;

  outw(0x90200004, 0x00151151);
  outw(0x90100018, 0x100b0000);
  data = inw(0x98100030);
  data = data | 0x00010000;
  outw(0x98100030, data);

  /* ag101 specific */
  data = inw(0x98100030);
  data = data | 0x00e00000;
  outw(0x98100030, data);
  data = inw(0x98100028);
  data = data | 0x00000010;
  outw(0x98100028, data);
  data = inw(0x9810004c);
  data = data | 0x0007f000;
  outw(0x9810004c, data);
  /* ag101 specific end */

  outw(0x90300000, 0x00011312);
  outw(0x90300004, 0x00480180);
  outw(0x90300008, 0x00002326);
  outw(0x9030000c, 0x00000010);

  // remap
  outw(0x90300010, 0x00001100);
  outw(0x90300014, 0x00000000);
  outw(0x90300018, 0x00000000);
  outw(0x9030001c, 0x00000000);
  outw(0x90300010, 0x00001000);

  /* ag101 specific */
  outw(0x90200008, 0x10000052);
  outw(0x9020000c, 0x000ff3ff);
  /* ag101 specific end */

  return 0x0;
}

/* Need send terminate to ICEman before quit Burner */
void terminate(void)
{
  char send_data[2];

  send_data[0] = 4;

  SEND(sock, send_data, 1);

#ifdef __MINGW32__
  closesocket (sock);
  do
  {
    WSACleanup ();
  } while (WSAGetLastError () != WSANOTINITIALISED);
#else
  close (sock);
#endif
}

void handle_int(int signo)
{
  terminate();
  fprintf(log, "\n!! burn not done yet !!\n");
  fflush(log);
  exit(0);
}

void initial_socket(unsigned int port)
{
  char ip[20] = "127.0.0.1";
  struct sockaddr_in server_addr;  

#ifdef __MINGW32__
  WSADATA wsadata;
  if (WSAStartup(MAKEWORD(1,1), &wsadata) == SOCKET_ERROR) {
      fprintf(log, "Error creating socket.");
      fflush(log);
      fflush(stdout);
  }
#endif

  if ((sock = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
     fprintf(log, "can't create socket\n");
     fflush(log);
     exit(1);
  }
  server_addr.sin_family = PF_INET;     
  server_addr.sin_port = htons(port);   
#ifdef __MINGW32__
  server_addr.sin_addr.s_addr = inet_addr( ip );
#else
  inet_aton(ip, &server_addr.sin_addr);
#endif 
  memset(&(server_addr.sin_zero),0,8); 
  if (connect(sock, (struct sockaddr *)&server_addr,
                    sizeof(struct sockaddr)) == -1) 
  {
    fprintf(log, "connecting fail !!\n");
    fflush(log);
    exit(1);
  }
}

unsigned char* get_image(FILE* image, unsigned int* size)
{
  unsigned char* flash;
  if (image == NULL)
  {
    fprintf (log, "Error: Burn image have to specify.\n");
    fflush(log);
    exit(1);
  }

  fseek(image, 0L, SEEK_END);
  (*size) = (ftell(image));
  fseek(image, 0L, SEEK_SET);

  if ((flash = (UINT8*)malloc((*size) * (sizeof(UINT8)))) == NULL) {
     fprintf(log, "Error: can't allocate memory (%d bytes) for flash file\n", (*size));
     fflush(log);
     return 0;
  }
  fread(flash, (*size), 1, image);
  return flash;
}


